import org.junit.Test;
import static org.junit.Assert.*;
import java.util.ArrayList;

public class TestWordList {
	private void add(WordList list, String[] values) {
		for (String s : values)
			list.insert(s);
	}
	protected WordList createList()	{
		return new SortedWordList();
	}
	protected WordList createList(String values) {
		WordList list = createList();
		if(values.length() > 0)
			add(list, values.split(" "));
		return list;
	}
	@Test
	public void testNode() throws Exception {
		WordNode n = new WordNode("Reinhard");
		assertTrue(n.count == 0 || n.count == 1);
		assertEquals(String.format("%s (%d times)",	n.word, n.count), 
			n.toString());
	}
	@Test
	public void testCount() throws Exception	{
		WordList list = createList(
			"Reinhard Wolfinger Reinhard Bilgeri Reinhard Berger");
		assertEquals(4, list.nOfDifferentWords());
		assertEquals(1, list.getFrequency("Berger"));
		assertEquals(1, list.getFrequency("Bilgeri"));
		assertEquals(3, list.getFrequency("Reinhard"));
		assertEquals(1, list.getFrequency("Wolfinger"));
		assertEquals(1.5, list.meanWordCount());
		assertEquals(3, list.getWords(1).length);
		assertEquals(1, list.getWords(3).length);
		assertEquals(2, list.getWordsStartingWith("b").length);
		list.remove("Reinhard");
		assertEquals(3, list.nOfDifferentWords());
		assertEquals(0, list.getFrequency("Reinhard"));
		assertEquals(0, list.getWords(3).length);
		assertEquals(3.0/3.0, list.meanWordCount(), 0.05);
	}
  @Test
  public void testSort() {
		WordList list = new SortedWordList();
		String[] names = { 
			"Friedrich", "Berta", "Anton", "Paula", "Caesar", "Zeppelin" };
		String[] sorted = { 
			"Anton", "Berta", "Caesar", "Friedrich", "Paula", "Zeppelin" };
		for(int i=0; i < names.length; i++)
			list.insert(names[i]);
		assertEquals(6, list.nOfDifferentWords());
		for(int i=0; i < names.length; i++)
			assertEquals(1, list.getFrequency(names[i]));
		String[] result = list.getWords(1);
		assertEquals(names.length, result.length);
		for(int i=0; i < names.length; i++)
			assertEquals(0 , result[i].compareToIgnoreCase(sorted[i]));
  }
	@Test
	public void testHamlet() {
		WordList list = createList("To be or not to be");
		assertEquals(2, list.getFrequency("be"));
		assertEquals(4, list.nOfDifferentWords());
		assertEquals(1.5, list.meanWordCount());
		assertEquals(2, list.getWords(2).length);
		assertEquals(0, list.getFrequency("notfound"));
		assertEquals(2, list.getFrequency("BE"));
		assertEquals(0, list.getWords(3).length);
	}
	@Test
	public void testEmpty()	{
		WordList list = createList("");
		assertEquals(0, list.nOfDifferentWords());
		assertEquals(0, list.getWords(1).length);
		assertEquals(0, list.getWordsStartingWith(("")).length);
	}
	@Test
	public void testClone() {
		WordList list1 = createList("One Two Three Four Five");
		assertEquals(5, list1.nOfDifferentWords());
		WordList list2 = list1.clone();
		assertEquals(5, list2.nOfDifferentWords());
		list2.remove("Two");
		list2.remove("Four");
		list1.insert("Six");
		assertEquals(3, list2.nOfDifferentWords());
		assertEquals(6, list1.nOfDifferentWords());
	}
	@Test
	public void testMerge() {
		WordList list1 = createList("One Two Three Four Five Six");
		assertEquals(6, list1.nOfDifferentWords());
		WordList list2 = createList("Five Six Seven Eight Nine");
		assertEquals(5, list2.nOfDifferentWords());
		WordList list3 = list1.clone();
		list3.merge(list2);
		assertEquals(6, list1.nOfDifferentWords());
		assertEquals(5, list2.nOfDifferentWords());
		assertEquals(9, list3.nOfDifferentWords());
		String[] words1 = list3.getWords(1);
		assertEquals(7, words1.length);
		assertTrue(words1[0].compareToIgnoreCase("Eight") == 0);
		assertTrue(words1[1].compareToIgnoreCase("Four") == 0);
		assertTrue(words1[2].compareToIgnoreCase("Nine") == 0);
		assertTrue(words1[3].compareToIgnoreCase("One") == 0);
		assertTrue(words1[4].compareToIgnoreCase("Seven") == 0);
		assertTrue(words1[5].compareToIgnoreCase("Three") == 0);
		assertTrue(words1[6].compareToIgnoreCase("Two") == 0);
		String[] words2 = list3.getWords(2);
		assertEquals(2, words2.length);
		assertTrue(words2[0].compareToIgnoreCase("Five") == 0);
		assertTrue(words2[1].compareToIgnoreCase("Six") == 0);
		list3.remove("Five");
		assertEquals(6, list1.nOfDifferentWords());
		assertEquals(5, list2.nOfDifferentWords());
		assertEquals(8, list3.nOfDifferentWords());
	}
	protected void readAlice(WordList list)	{
    In.open("alice.txt");
		assertTrue(In.done());
    ArrayList words = new ArrayList();
    char ch = In.peek();
    while (ch != In.eof) {
      if (Character.isLetter(ch)) {
        String ident = In.readIdentifier();
        words.add(ident);
      }
      else {
        In.read();
      }
      ch = In.peek();
    }
		assertEquals(27575, words.size());
    // Test inserting words
    for (int j = 0; j < words.size(); j++)
      list.insert((String)words.get(j));
		In.close();
	}
	@Test	
	public void testAlice()	{
		WordList list = new SortedWordList();
		readAlice(list);
		assertEquals(2601, list.nOfDifferentWords());
		assertEquals(1648, list.getFrequency("the"));
		assertEquals(417, list.getFrequency("alice"));
		assertEquals(5, list.getFrequency("home"));
		assertEquals(3, list.getFrequency("love"));
		assertEquals(0, list.getFrequency("humpty"));
		assertEquals(10.6, list.meanWordCount(), 0.05);
		assertEquals(179, list.getWordsStartingWith("t").length);
		assertEquals(99, list.getWordsStartingWith("e").length);
		assertEquals(142, list.getWordsStartingWith("a").length);
	}
}
